
set debug 0
set started 0

loadsource colorpick

set frame OPT_COLOURS
set frametitle "Colour Selection Frame"
set icontitle "ColourSel"

set savefile [file join $env(HOME) .eg-tkcolours.tcl]

set wtype 0
set wcomp 0
set windowtype ""
set windowcomponent ""

set colour ""
set palette ""

set widgets [list button checkbutton entry frame label listbox menubutton menu message \
  scale scrollbar text canvas spectrum]
set components [list activebackground activeforeground background \
  disabledforeground foreground highlightbackground highlightcolor \
  insertbackground selectcolor selectbackground selectforeground \
  troughcolor tagcolour boxcolour activeboxcolour textcolour activetextcolour gatecolour polycolour \
  pointercolour speccolour]

set spectrumcomponents [list background foreground tagcolour boxcolour activeboxcolour \
  textcolour activetextcolour gatecolour polycolour pointercolour speccolour]

set buttoncomponents [list activebackground activeforeground background \
  disabledforeground foreground highlightbackground highlightcolor ]

set checkbuttoncomponents [list activebackground activeforeground background \
   disabledforeground foreground highlightbackground highlightcolor selectcolor]

set entrycomponents [list background foreground highlightbackground \
   highlightcolor insertbackground selectbackground selectforeground]

set framecomponents [list background highlightbackground highlightcolor]

set labelcomponents [list background foreground highlightbackground \
   highlightcolor]

set menucomponents [list activebackground activeforeground background \
   foreground disabledforeground]

set menubuttoncomponents [list activebackground activeforeground background \
   foreground disabledforeground]

set messagecomponents [list background foreground highlightbackground \
   highlightcolor ]

set scalecomponents [list activebackground background foreground \
   highlightbackground highlightcolor troughcolor]

set scrollbarcomponents [list activebackground background highlightbackground \
   highlightcolor troughcolor]

set textcomponents [list background foreground highlightbackground \
   highlightcolor insertbackground selectbackground selectforeground]

set listboxcomponents [list background foreground highlightbackground \
   highlightcolor selectbackground selectforeground]

set canvascomponents [list background highlightcolor highlightbackground \
   insertbackground selectbackground selectforeground]


proc make-frame {} {
  global env wtype widgets wcomp

  :set footer_position center
  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
    :icon-name "[self icon-title]" :resize F

  :panel COLORSELECTION :padx 5 :pady 5 :layout L

  :panel P2 :padx 10 :pady 10 :layout L
  :layout V
  :row-gap 5
  :choice-stack WTYPE :valuex 80 :value $wtype :label "window type" :strings $widgets
  :choice-stack WCOMP :valuex 80 :value $wcomp :label "component"

  :text WVALUE :valuex 70 :label "colour" :value "" :w 20
  :text WPALETTE :valuex 70 :label "palette" :value "" :w 20
  :vskip 25

  :layout H
  :col-gap 10
  :next-row T
  :button RESTORE  :label "Restore Settings"
  :button SAVE     :label "Save Settings as default"

  :next-row T
  :button RESET     :label "Reset"
  :button APPLY     :label "Apply"
  :button REDISPLAY :label "Repaint"

  :next-row T
  :button PALETTE :label "Select Palette"
  :button SELECT  :label "Select Colour"
  :button CHOOSE  :label "Choose Colour"
  if {$env(platform) != "windows"} then {:modify CHOOSE :inactive T}
  if {[info tclversion] < "8.0"} then {:modify CHOOSE :inactive T}
}

proc paint-sample {} {
   global _wreq

  set menu {
    {"The first item in this menu"}
    {"An example of a cascade" "first item in the cascade" "second item" "and the last item"}
    {"The second and last item in this menu"}
  }

 :modify [self frame-name] :closed T

 catch {:modify S1 :delete T}
 catch {:modify S2 :delete T}
 catch {:modify S3 :delete T}
 catch {:modify S4 :delete T}

 :panel S1 :padx 10 :pady 10 :layout B :after FOOTER
 :layout H; :row-gap 5; :col-gap 25

 :button SB   :label "A Sample Button"
 :slider SS   :label "A Sample Slider" :value 1 :min 0 :max 9
 :next-row T

 :message SM  :label "A Sample Message"
 :gauge SGG    :label "A Sample Gauge" :value 1 :min 0 :max 9

 :panel S2 :padx 10 :layout B :after FOOTER
 :layout V
 :row-gap 5; :col-gap 10

 :number SN   :label "A Sample Number" :value 1 :min 0 :max 9
 :choice-stack SCS :label "A Sample Choice-Stack" :strings [list c1 c2]
 :choice SCH  :label "A Sample Choice"  :value 1 :strings [list 1 2] 
 :menubutton SMB :label "A Sample MenuButton" :menu $menu
 :text ST      :label "A Sample Text"     :value "Midas Forever"
 :checkbox SCB :label "A Sample Checkbox" :value 0
 
 global _rowgap _colgap
 :galley SG  :strings "A Sample Galley (text widget)"  :rows 4 :cols 30 \
     :x [expr [winfo width [:path ST]] + $_colgap] \
     :y 0

 :canvas SC  :w 200 :h 100 \
     :x [expr [winfo width [:path ST]] + $_colgap] \
     :y [expr [winfo height [:path SG]] + $_rowgap]

 :draw text :text "A Sample Canvas" :colour red :value 50 70
 :draw oval :colour blue :value 15 15 185 65

 :panel S3 :padx 10 :pady 5 :layout B :after FOOTER
 :list  SL :strings [list "A Sample List (listbox widget)"] :rows 2 :cols 25
 :panel S4 :padx 10 :layout B :after FOOTER :h 100

 if {[catch {:spectrum SPEC spectrum create}] == 0} then {
   :spectrum SPEC spectrum add _inbuilt1d_
   after 1000 :spectrum SPEC tag set 200 50
 }

 :modify [self frame-name] :closed F

 set-footer "Sample repainted at [date]"
}

proc paint-colour {} {
  global env widgets
  global wtype wcomp palette colour
  set type [lindex $widgets $wtype]
  global [set type]components
  set comp [eval lindex $[set type]components $wcomp]

  global _colour_[set type]_[set comp]
  if { [info exists _colour_[set type]_[set comp]] } then {
    set colour [set _colour_[set type]_[set comp]]
  } else {
    if {[info exists env(tk$type$comp)]} then {
      set colour $env(tk$type$comp)
    } else {
      set colour ""
    }
  }
  :modify WVALUE :value $colour
  :modify WPALETTE :value "$palette"

#    enter selection into the "old" selector panel
  global data
  set data(selection) $colour
  tkColorDialog_HandleSelEntry [:path COLORSELECTION]
}

proc set-eg-option {k v} {
   global env
   set ll {}
   foreach p $env(EGOPTIONS) {
      if {[first $p]!=$k} then {lappend ll $p}
   }
   lappend ll [list $k $v]
   set env(EGOPTIONS) $ll
}

proc select_colour {c} {
  global colour
  set colour $c
}

proc click-palette {} {
  global data
  global colour palette
  set colour $data(selection)
  set palette $colour
  tk_setPalette [set colour]
  paint-sample
}

proc click-select {} {
  global data
  global widgets
  global wtype wcomp colour
  set colour $data(selection)
  set type [lindex $widgets $wtype]
  global [set type]components
  set comp [lindex [set [set type]components] $wcomp]
  :colour local $type $comp $colour
}

proc click-choose {} {
   global windowtype windowcomponent colour
   global widgets wtype wcomp
   set c [tk_chooseColor -title "Choose a colour for $windowtype $windowcomponent" \
      -parent [:path COLORSELECTION] -initialcolor $colour]
   if {$c != ""} then {
      set colour $c
      set type [lindex $widgets $wtype]
      global [set type]components
      set comp [lindex [set [set type]components] $wcomp]
      :colour local $type $comp $colour
   }
}

proc click-wtype {v} {
  global widgets wtype wcomp windowtype windowcomponent
  set wtype $v
  set windowtype [lindex $widgets $wtype]
  global [set windowtype]components
  set windowcomponent [lindex [set [set windowtype]components] $wcomp]
  :modify WCOMP :strings [set [set windowtype]components] :value $wcomp
}

proc click-wcomp {v} {
  global widgets wtype wcomp  windowtype windowcomponent
  set wcomp $v
  global [set windowtype]components
  set windowcomponent [lindex $[set windowtype]components $wcomp]
  :modify WCOMP :value $wcomp
}

proc click-restore {} {
  global palette

   click-reset

   colour-config default
   inform root exec-colour-defaults
   colour-setup
   set palette [get-eg-option tkpalette]
}

proc click-reset {} {
  global started widgets components

   foreach widget $widgets {
     foreach comp $components {
        global _colour_[set widget]_[set comp]
        catch {unset _colour_[set widget]_[set comp]}
     }
   }
   colour-setup
   set started 0; start
}

proc click-apply {} {
  global env started widgets components palette
  global windowtype wcomp
  foreach widget $widgets {
    foreach comp $components {
      global _colour_[set widget]_[set comp]
      if { [info exists _colour_[set widget]_[set comp]] } then {
        set env(tk$widget$comp) [set _colour_[set widget]_[set comp]]
      }
    }
  }
  colour-setup
  set started 0; start
}

proc click-save {} {
  global env savefile palette widgets components
  set z [catch {open $savefile w} file]
  if {$z != 0} then {set-footer $file; return}
  puts $file "set-eg-option tkpalette $palette"
  puts $file "set env(USERCOLORS) 1"
  foreach widget $widgets {
    foreach comp $components {
      global _colour_[set widget]_[set comp]
      if { [info exists _colour_[set widget]_[set comp]] } then {
        set colour [set _colour_[set widget]_[set comp]]
      } else {
        if {[info exists env(tk$widget$comp)]} then {
          set colour $env(tk$widget$comp)
        } else {
          set colour ""
        }
      }
    if {$colour != ""} then {puts $file "set env(tk$widget$comp) $colour"}
    }
  }
  close $file
}


proc click arg {
   global debug
   global wtype wcomp

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      PALETTE   {click-palette}
      SELECT    {click-select}
      CHOOSE    {click-choose}

      WTYPE     {set wcomp 0; click-wtype $v}
      WCOMP     {click-wcomp $v}

      RESTORE   {click-restore; paint-sample}
      SAVE      {click-save}

      RESET     {click-reset; paint-sample}
      APPLY     {click-apply}
      REDISPLAY {paint-sample}

      default   {}
   }
   paint-colour
}


proc start args {
   global started
   global palette wtype
   if {$started == 0} then {
      make-frame
      click-wtype $wtype
      set palette [get-eg-option tkpalette]
      tkColorDialog [:path COLORSELECTION]
      set started 1
   }
   open-frame
   paint-sample
   paint-colour
}
