# V814.tcl---CAEN V814 16 Ch LTD module control frame

set frame "V814"
set frametitle "CAEN V814 16 Channel LTD"
set icontitle "V814"

set modulename v814
set moduletype  "CAEN V814"

loadsource common.vme

set maxadc 16
set adc  0

set alladcs 0
set allmodules 0


for {set i 0} {$i < $maxadc} {incr i} {
   set w2r(THRESHOLD$i)   Threshold$i
   set adc2r(THRESHOLD$i) Threshold\[?,??\]
   set w2r(ENABLE$i)     Enable$i
   set adc2r(ENABLE$i)   Enable\[?,??\]
}
set w2r(OUTPUTWIDTH07)  OutputWidthL
set w2r(OUTPUTWIDTH815) OutputWidthH
set w2r(TEST)           Test
set w2r(MAJTHR)         MajorityThreshold
#set w2r(ENABLE)         Enable

proc click-widget {w value} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc click-adc-widget {w value} {
    global w2r adc2r
    global modulename module
    global alladcs allmodules

    if {$alladcs == 0} then {set register $w2r($w)} else {set register $adc2r($w)}

    if {$allmodules == 0} then {
        write-vme-registers $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc click-select-module {v} {
   global module modulename maxmodule
   global CurrentVMEServer

   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   set CurrentVMEServer [VMEmodule2server [set modulename]#[set module]]
   click-redisplay
}

proc click-all-adcs {v} {global alladcs; set alladcs $v}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc click-majthr {vl} {
   set majthr [expr ((($vl*50)-25)+2)/4]
   click-widget MAJTHR $majthr
}

proc click-ID {} {
    global modulename module
    foreach id {1 2 3} {
       set vl [read-vme-register $modulename#$module.ModID#$id]
       insert-log "V814 Module Identifier Word $id = $vl"
    }
}

proc redisplay-majthr {} {
    global modulename module w2r
    set register $w2r(MAJTHR)
    set value [read-vme-register $modulename#$module.$register]
    :modify MAJTHR :value [expr ((($value*4)+25)+25)/50]
}
   
proc redisplay-module {} {
   global w2r modulename module maxmodule

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   foreach w {OUTPUTWIDTH07 OUTPUTWIDTH815} {
       set register $w2r($w)
       set value [read-vme-register $modulename#$module.$register]
       :modify $w :value $value
   }
   
   redisplay-majthr
}

proc redisplay-adcs {} {
    global w2r
    global modulename module maxadc

    foreach W {THRESHOLD ENABLE} {
       for {set i 0} {$i < $maxadc} {incr i} {
          set w $W$i
          set register $w2r($w)
          set value [read-vme-register $modulename#$module.$register]
          :modify $w :value $value
       }
    }
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module
   redisplay-adcs

   clear-busy
   set EGreport 0
}


proc click arg {
   global debug
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      OUTPUTWIDTH07   -
      OUTPUTWIDTH815  -
      TEST        {click-widget $w $v}
      
      ID          {click-ID}
      MAJTHR      {click-majthr $v}

      ENABLE*     -
      THRESHOLD*  {click-adc-widget $w $v}

      REDISPLAY      {click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env ac
  global module maxmodule modulelist maxadc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :hskip 80
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T
     :message M1 :label "Module Setup" :bold T
     :next-row T
     :col-gap 10

     :slider OUTPUTWIDTH07    :label "Output Pulse Width (Ch 0=>7)"    :min 0 :max 255 :valuex 32c
     :slider OUTPUTWIDTH815   :label "Output Pulse Width (Ch 8=>15)"   :min 0 :max 255 :valuex 32c
     :next-row T
     
     :slider MAJTHR    :label "Majority Threshold" :min 1 :max 20 :valuex 32c
  
   :panel ADC :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 0
     :message M2 :label "Channel Enables and Thresholds" :bold T
     :hskip 400
     :checkbox ALL-ADCS :label "Act on ALL Channels?"
     :next-row T

     for {set i 0} {$i < $maxadc} {} {
         :checkbox ENABLE$i    :label $i :valuex 2c 
         :slider   THRESHOLD$i :label "" :min 0 :max 255 :entry-width 3
         :hskip 10
         incr i
         if {[expr $i / 4 * 4] == $i} {:next-row T}
     }
  
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :button TEST :label "Generate Test Pulse"
      :button ID   :label "Identify Module"

  :show T

}
