
#----------------------------------------------------------------------BF
set BF(frame-name) BF
set BF(frame-title) "Birmingham Fit Options"
set BF(icon-title) optBF

set BF(shapenames) {gauss skew skewtail lorentz}
set BF(widthoptnames) {different same given}
set BF(peakoptnames) {different given}
set BF(ordernames) {constant linear quadratic cubic quartic quintic}
set BF(shape) gauss
set BF(widthopt) different
set BF(peakopt) different
set BF(widths) {5.0}
set BF(skews) {10.0}
set BF(order) linear
#set BF(order) quadratic


proc BF.make-frame {} { 
   global env BF

   catch {BF.unpack}

   :frame $BF(frame-name) :label "$BF(frame-title)" :show-footer F :toplevel T \
        :icon-image "@$env(ICONHOME)/spectrum"  \
           :icon-name $BF(icon-title)  :resize F

   :panel BF-OPTIONS :layout T :padx 5 :pady 5
    :layout H; :row-gap 5

   :message    BF-L1  :label "Peak shape:"      :valuex 15c
   :menubutton BF-SHAPE    :label "$BF(shape)"  :w 15 :strings $BF(shapenames)  :command "BF.opt shape"
   :next-row T
   :message    BF-L2  :label "Width treatment:" :valuex 15c  
   :menubutton BF-WOPT :label "$BF(widthopt)" :w 15 :strings $BF(widthoptnames) :command "BF.opt widthopt"
   :next-row T
   :message    BF-L3  :label "Peak treatment:"  :valuex 15c
   :menubutton BF-POPT  :label "$BF(peakopt)" :w 15 :strings $BF(peakoptnames) :command "BF.opt peakopt"
   :next-row T
   :message    BF-L4  :label "Background poly:" :valuex 15c
   :menubutton BF-ORDER    :label "$BF(order)"  :w 15 :strings $BF(ordernames)  :command "BF.opt order"
   :next-row T
   :text    BF-WIDTH :label "Peaks FWHMs:" :valuex 16c :w 15 :value $BF(widths) :command "BF.optX widths"
   :next-row T
   :text    BF-SKEW  :label "Skew FWHMs:"  :valuex 16c :w 15 :value $BF(skews)  :command "BF.optX skews"

   :panel BF-BUTTONS :layout B :padx 5
    :layout H
   :button BF-SAVE  :label "Save as defaults" :command BF.save
    :hskip 75
   :button BF-HELP  :label Help  :command click-help

   :show T
}

proc BF.opt {opt args} {
   global BF
   set BF($opt) [second $args]
   :modify [first $args] :label [second $args]
   BF.announce
}

proc BF.optX {opt args} {
   global BF
   set BF($opt) [second $args]
   BF.announce
}

proc BF.unpack {} {
   global BF
   foreach p [get-eg-option brumfit] {
      switch [first $p] {
      shape {set BF(shape) [second $p]}
      widthopt {set BF(widthopt) [second $p]}
      peakopt {set BF(peakopt) [second $p]}
      widths {set BF(widths) [second $p]}
      skews {set BF(skews) [second $p]}
      order {set BF(order) [lindex $BF(ordernames) [second $p]]}
      }
   }
}

proc BF.announce {} {
   global BF
   set-eg-option brumfit [list \
        [list shape $BF(shape)] \
        [list widthopt $BF(widthopt)] \
        [list peakopt $BF(peakopt)] \
        [list order [lsearch $BF(ordernames) $BF(order)]] \
        [list widths $BF(widths)] \
        [list skews $BF(skews)]]
}

proc BF.save {} {edit-defaults brumfit "set-eg-option brumfit \{[get-eg-option brumfit]\}"}
